@@ -12,12 +12,11 @@ from .base import NAMES, constant
 from .error import (
     ProjectDataMissingError,
     TemplateDataMissingError,
-    ValidationError,
 )
 
 if TYPE_CHECKING:
-    from .base import Modes
-    from typing import Optional, Tuple, List, Dict, Iterable
+    from .base import LinkMode
+    from typing import Optional, List, Dict
 
 
 def toml_load(path_obj: Path, missing_ok: bool = False) -> Dict:
@@ -64,7 +63,7 @@ class Config:
         """TODO: write"""
         self.working_dir = working_dir
         self._dct = _merge(
-            toml.loads(resources.read_text(defaults, "config.toml")),
+            DefaultLoader.config(),
             toml_load(self.global_path, missing_ok=True),
             toml_load(self.local_path, missing_ok=True),
         )
@@ -111,52 +110,47 @@ class _JinjaTemplatePath:
 
     def template_doc(self, name: str) -> Path:
         """TODO: write"""
-        return Path("templates", name, NAMES.template_doc)
+        return Path(name, NAMES.template_doc)
 
     @constant
-    def _template_resource_dir(self) -> Path:
+    def project_macro(self) -> str:
         """TODO: write"""
-        return Path("resources", "other")
+        return "project_macro_file.tex"
 
     @constant
-    def project_macro(self) -> Path:
+    def gitignore(self) -> str:
         """TODO: write"""
-        return self._template_resource_dir / "project_macro_file.tex"
+        return "gitignore"
 
     @constant
-    def gitignore(self) -> Path:
+    def build_latex(self) -> str:
         """TODO: write"""
-        return self._template_resource_dir / "gitignore"
+        return "build_latex.yml"
 
     @constant
-    def build_latex(self) -> Path:
+    def pre_commit(self) -> str:
         """TODO: write"""
-        return self._template_resource_dir / "build_latex.yml"
+        return "pre-commit"
 
     @constant
-    def pre_commit(self) -> Path:
+    def classinfo(self) -> str:
         """TODO: write"""
-        return self._template_resource_dir / "pre-commit"
+        return "classinfo.tex"
 
     @constant
-    def classinfo(self) -> Path:
+    def bibinfo(self) -> str:
         """TODO: write"""
-        return self._template_resource_dir / "classinfo.tex"
+        return "bibinfo.tex"
 
     @constant
-    def bibinfo(self) -> Path:
+    def bibliography(self) -> str:
         """TODO: write"""
-        return self._template_resource_dir / "bibinfo.tex"
+        return "bibliography.tex"
 
     @constant
-    def bibliography(self) -> Path:
+    def arxiv_autotex(self) -> str:
         """TODO: write"""
-        return self._template_resource_dir / "bibliography.tex"
-
-    @constant
-    def arxiv_autotex(self) -> Path:
-        """TODO: write"""
-        return self._template_resource_dir / "arxiv_autotex.txt"
+        return "arxiv_autotex.txt"
 
 
 def relative(base: str):
@@ -197,26 +191,6 @@ class ProjectPath:
         self.config = Config(working_dir)
         self.name = self.dir.name
 
-    def validate(self, exists=True) -> None:
-        """TODO: write"""
-        if not exists and any(path.exists() for path in self.rootfiles):
-            raise ValidationError(
-                f"conflicting project files already exist in the working directory."
-            )
-        if exists and any(not path.exists() for path in self.minimal_files):
-            raise ValidationError(f"the working directory is not a valid project.")
-
-    def validate_git(self, exists=True) -> None:
-        """TODO: write"""
-        if not exists and any(path.exists() for path in self.gitfiles):
-            raise ValidationError(
-                f"conflicting git files already exist in the working directory."
-            )
-        if exists and any(not path.exists() for path in self.minimal_gitfiles):
-            raise ValidationError(
-                f"the working directory is not a valid git repository."
-            )
-
     @relative("data")
     def template(self) -> str:
         """TODO: write"""
@@ -287,26 +261,6 @@ class ProjectPath:
         """TODO: write"""
         return "pre-commit"
 
-    @constant
-    def gitfiles(self) -> Tuple[Path, Path, Path]:
-        """TODO: write"""
-        return (self.pre_commit, self.github_home, self.gitignore)
-
-    @constant
-    def minimal_gitfiles(self) -> Tuple[Path]:
-        """TODO: write"""
-        return (self.git_home,)
-
-    @constant
-    def rootfiles(self) -> Tuple[Path, Path, Path]:
-        """TODO: write"""
-        return (self.main, self.project_macro, self.data_dir)
-
-    @constant
-    def minimal_files(self) -> Tuple[Path, Path]:
-        """TODO: write"""
-        return (self.main, self.data_dir)
-
     def mk_data_dir(self):
         for mode in NAMES.modes:
             (self.data_dir / NAMES.resource_subdir(mode)).mkdir(
@@ -345,17 +299,22 @@ class _BaseLinker:
 class _FileLinker(_BaseLinker):
     """TODO: write"""
 
-    def __init__(self, suffix: str, user_str: str, mode: Modes):
+    def __init__(self, suffix: str, user_str: str, mode: LinkMode):
         """TODO: write"""
         super().__init__(
             DATA_PATH.resource_dir / NAMES.resource_subdir(mode), suffix, user_str
         )
-        self.mode = mode  # type: Modes
+        self.mode = mode  # type: LinkMode
 
 
-def _load_default_template() -> Dict:
-    """TODO: write"""
-    return toml.loads(resources.read_text(defaults, "template.toml"))
+class DefaultLoader:
+    @staticmethod
+    def template() -> Dict:
+        return toml.loads(resources.read_text(defaults, "template.toml"))
+
+    @staticmethod
+    def config() -> Dict:
+        return toml.loads(resources.read_text(defaults, "config.toml"))
 
 
 def toml_load_local_template(path: Path) -> Dict:
@@ -366,7 +325,7 @@ def toml_load_local_template(path: Path) -> Dict:
         raise ProjectDataMissingError(
             path, message="The local template file is missing."
         ) from err
-    return _merge(_load_default_template(), template)
+    return _merge(DefaultLoader.template(), template)
 
 
 def toml_load_system_template(path: Path, user_str: str, name: Optional[str] = None):
@@ -375,7 +334,7 @@ def toml_load_system_template(path: Path, user_str: str, name: Optional[str] = N
         template = toml_load(path)
     except FileNotFoundError as err:
         raise TemplateDataMissingError(path, user_str=user_str, name=name) from err
-    return _merge(_load_default_template(), template)
+    return _merge(DefaultLoader.template(), template)
 
 
 class _TemplateLinker(_BaseLinker):
